/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.isobus.imp;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.Icon;
import nl.cloudfarming.client.imp.ImportProcessor;
import nl.cloudfarming.client.isobus.type.TaskDataObject;
import nl.cloudfarming.client.logging.AppLogFactory;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 * Isobus implementation of {@link ImportProcessor}
 *
 * <p>Provides a hook into the generic data import system. This implementation
 * adds support for isobus TASKDATA.XML and the connected binary files. The {@link IsobusImportProcessor#doImport(org.openide.filesystems.FileObject, org.netbeans.api.project.Project)
 * } method will copy all files which belong to the provided taskdata file to a
 * new folder withing import/isobus.</p>
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@NbBundle.Messages(
        { "isobus importer failed import=Isobus taskdata import failed, please check the log file for more detailed information."
        , "isobus importer succesfull import=Isobus taskfile imported successfully"
        , "isobus taskdata format description=Isobus data (TASKDATA.XML)"}
                )
@ServiceProvider(service = ImportProcessor.class)
public class IsobusImportProcessor implements ImportProcessor {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.isobus.type.IsobusImportProcessor");
    private static final String IMPORTED_TASKFILES_FOLDER = "Isobus";
    private static final Icon ICON = ImageUtilities.loadImageIcon("nl/cloudfarming/client/isobus/imp/iso_add.png", true);

    @Override
    public String[] getSupportedExtentions() {
        return new String[]{"XML", "xml"};
    }

    @Override
    public boolean isSupported(FileObject importFile) {
        return "xml".equalsIgnoreCase(importFile.getExt()) && "taskdata".equalsIgnoreCase(importFile.getName());
        //FIXME don't create dataobjects in the isSupported method since it will also be used in file pickers
//        try {
//            DataObject dataObject = DataObject.find(importFile);
//            //FIXME there should be something in the lookup of the task data data object
//            // so we don't have to use instanceof
//            if (dataObject != null && dataObject instanceof TaskDataObject) {
//                return true;
//            }
//        } catch (DataObjectNotFoundException ex) {
//        }
//        return false;
    }

    @Override
    public String getFormatDescription() {
        return Bundle.isobus_taskdata_format_description();
    }

    @Override
    public Icon getIcon() {
        return ICON;
    }

    @Override
    public void doImport(FileObject importFile, Project project) throws IllegalArgumentException {
        assert isSupported(importFile) : " unsupported import file provided to doImport";
        try {
            DataObject taskDataObject = DataObject.find(importFile);
            FileObject isobusFolder = getIsobusImportFolder(project);
            FileObject destination = isobusFolder.createFolder(FileUtil.findFreeFolderName(isobusFolder, importFile.getName()));
            copySourceFiles(taskDataObject, destination);
            AppLogFactory.getLogger(IsobusImportProcessor.class).info(Bundle.isobus_importer_succesfull_import());
        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Import of isobus taskfile {0} failed with exception {1}", new Object[]{importFile.getPath(), ex.getMessage()});
            AppLogFactory.getLogger(IsobusImportProcessor.class).error(Bundle.isobus_importer_failed_import());
        }

    }

    @Override
    public void process(FileObject importFile, Project project) throws IllegalArgumentException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public <T> void process(FileObject importFile, Project project, List<T> toPopulate) throws IllegalArgumentException {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    /**
     * {
     *
     * @inheritDoc }
     */
    @Override
    public Node getImportNode(Project project) {
        try {
            FileObject shapesImportFolder = getIsobusImportFolder(project);
            DataFolder dataFolder = DataFolder.findFolder(shapesImportFolder);
            assert dataFolder != null;
            return new IsobusRootNode(project, dataFolder.getNodeDelegate());

        } catch (IOException ex) {
            LOGGER.log(Level.SEVERE, "Could not find or create folder {0} with the exception {1}", new Object[]{IMPORTED_TASKFILES_FOLDER, ex});
            Exceptions.printStackTrace(ex);
            return null;
        }
    }

    FileObject getIsobusImportFolder(Project project) throws IOException {
        FileObject projectImportFolder = Util.getImportFolder(project);
        FileObject isobusImportFolder = projectImportFolder.getFileObject(IMPORTED_TASKFILES_FOLDER);
        if (isobusImportFolder == null) {
            isobusImportFolder = projectImportFolder.createFolder(IMPORTED_TASKFILES_FOLDER);

        }
        return isobusImportFolder;
    }

    //FIXME move to util
    private void copySourceFiles(DataObject dataObject, FileObject targetFolder) {
        Iterator<FileObject> files = dataObject.files().iterator();
        while (files.hasNext()) {
            FileObject current = files.next();
            try {
                String newFileName = FileUtil.findFreeFileName(targetFolder, current.getName(), current.getExt());
                FileUtil.copyFile(current, targetFolder, newFileName);
            } catch (IOException ex) {
                LOGGER.log(Level.SEVERE, "Could not copy file {0} with the exception {1}", new Object[]{current.getName(), ex});
            }
        }
    }
}
